/*
 * =====================================================
 *  _____                _     ____  _____ _   _ _   _ 
 * |_   _|_ __ _   _ ___| |_  / __ \|  ___| | | | | | |
 *   | | | '__| | | / __| __|/ / _` | |_  | |_| | |_| |
 *   | | | |  | |_| \__ \ |_| | (_| |  _| |  _  |  _  |
 *   |_| |_|   \__,_|___/\__|\ \__,_|_|   |_| |_|_| |_|
 *                            \____/                   
 *
 * =====================================================
 *
 * This software is Copyright (C) 2006-2011
 * Fachhochschule Hannover (University of Applied Sciences and Arts)
 * Use is subject to license conditions.
 * 
 * The main licensing options available are:
 * 
 * Open Source Licensing. This is the appropriate option if you want to
 * share the source code of your application with everyone you
 * distribute it to, and you also want to give them the right to share
 * who uses it. If you wish to use TNC@FHH under Open Source Licensing,
 * you must contribute all your source code to the open source
 * community in accordance with the GPL Version 2 when your application
 * is distributed. See http://www.gnu.org/licenses/gpl-2.0.html/ or gpl-2.0.txt
 * 
 * Commercial Licensing. This is the appropriate option if you are
 * creating proprietary applications and you are not prepared to
 * distribute and share the source code of your application.
 * Contact trust@f4-i.fh-hannover.de for details.
 * 
 * http://trust.inform.fh-hannover.de/
 * 
 */

package de.fhhannover.inform.trust.xacml;

import java.io.File;

import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;

/**
 * This class starts a XACML PDP that will refresh its policies after a given
 * time. It can recognize changes on existing policies as well as newly added
 * policies or policies, that were deleted and therefore shall not be used
 * further.
 * 
 * @author Bastian Hellmann
 */
public class DynamicNetworkPDPStarter {

	private static Logger logger = Logger
			.getLogger(StaticNetworkPDPStarter.class);

	/**
	 * Starts the Trust XACML PDP. Uses the {@link DynamicNetworkPDP} to handle
	 * the policies.
	 * 
	 * @param args
	 *            expects four parameters: 1. port, 2. refresh time for
	 *            policies, 3. filename extension for policy files and 4. path
	 *            to policy files to use
	 */
	public static void main(String[] args) {
		PropertyConfigurator.configure("log4j.properties");

		if (args.length != 4) {
			System.err.println("Wrong number of arguments");
			System.err.println("Usage:");
			System.err
					.println("DynamicNetworkPDPStarter <port> <refresh time for policies> <filename postfix, e.g. '.xml'> <path to policy files (must end with defined filename postfix')>");
			System.exit(1);
		} else {
			logger.info("Trust@FHH XACML PDP with (dynamic policy updating) starts.");

			int port = Integer.parseInt(args[0]);
			long refreshTime = Long.parseLong(args[1]);
			String filenamePostfix = args[2].replace(".", "");
			String directoryPath = args[3];

			File policyDirectory = new File(directoryPath);
			if (policyDirectory.isDirectory()) {
				logger.info("Loading all files ending with '."
						+ filenamePostfix + "' from directory "
						+ policyDirectory);

				DynamicNetworkPDP pdp = new DynamicNetworkPDP(policyDirectory,
						filenamePostfix, refreshTime);

				NetworkPDPDispatcher dispatcher;
				dispatcher = new NetworkPDPDispatcher(pdp, port);

				logger.info("Dispatcher is started.");

				dispatcher.runServer();
			} else {
				logger.error("Could'nt load directory: " + directoryPath);
				System.exit(1);
			}
		}
	}
}
