/*
 * =====================================================
 *  _____                _     ____  _____ _   _ _   _ 
 * |_   _|_ __ _   _ ___| |_  / __ \|  ___| | | | | | |
 *   | | | '__| | | / __| __|/ / _` | |_  | |_| | |_| |
 *   | | | |  | |_| \__ \ |_| | (_| |  _| |  _  |  _  |
 *   |_| |_|   \__,_|___/\__|\ \__,_|_|   |_| |_|_| |_|
 *                            \____/                   
 *
 * =====================================================
 *
 * This software is Copyright (C) 2006-2011
 * Fachhochschule Hannover (University of Applied Sciences and Arts)
 * Use is subject to license conditions.
 * 
 * The main licensing options available are:
 * 
 * Open Source Licensing. This is the appropriate option if you want to
 * share the source code of your application with everyone you
 * distribute it to, and you also want to give them the right to share
 * who uses it. If you wish to use TNC@FHH under Open Source Licensing,
 * you must contribute all your source code to the open source
 * community in accordance with the GPL Version 2 when your application
 * is distributed. See http://www.gnu.org/licenses/gpl-2.0.html/ or gpl-2.0.txt
 * 
 * Commercial Licensing. This is the appropriate option if you are
 * creating proprietary applications and you are not prepared to
 * distribute and share the source code of your application.
 * Contact trust@f4-i.fh-hannover.de for details.
 * 
 * http://trust.inform.fh-hannover.de/
 * 
 */

package de.fhhannover.inform.trust.xacml;

import java.io.IOException;
import java.net.ServerSocket;

import org.apache.log4j.Logger;

/**
 * 
 * 
 * @author Bastian Hellmann
 */
public class NetworkPDPDispatcher { // extends SimplePDP {

	private static Logger logger = Logger.getLogger(NetworkPDPDispatcher.class);

	private NetworkPDP pdp;

	private int port;

	private final int DEFAULT_PORT = 12345;

	public NetworkPDPDispatcher(NetworkPDP pdp) {
		super();
		this.pdp = pdp;
		this.port = DEFAULT_PORT;
	}

	public NetworkPDPDispatcher(NetworkPDP pdp, int port) {
		super();
		this.pdp = pdp;
		this.port = port;
	}

	public void runServer() {
		boolean listening = true;
		ServerSocket serverSocket = null;

		try {
			serverSocket = new ServerSocket(this.port);
			logger.info("Server listening on port: " + this.port);

			int number = 1;
			while (listening) {
				new XACMLWorker(this.pdp, serverSocket.accept(), number)
						.start();
				logger.info("XACMLWorker #" + number + " starts.");
				number++;
			}

			serverSocket.close();
		} catch (IOException e) {
			logger.error("Could not listen on port: " + this.port);
			System.exit(1);
		}
	}
}
