/*
 * =====================================================
 *  _____                _     ____  _____ _   _ _   _ 
 * |_   _|_ __ _   _ ___| |_  / __ \|  ___| | | | | | |
 *   | | | '__| | | / __| __|/ / _` | |_  | |_| | |_| |
 *   | | | |  | |_| \__ \ |_| | (_| |  _| |  _  |  _  |
 *   |_| |_|   \__,_|___/\__|\ \__,_|_|   |_| |_|_| |_|
 *                            \____/                   
 *
 * =====================================================
 *
 * This software is Copyright (C) 2006-2011
 * Fachhochschule Hannover (University of Applied Sciences and Arts)
 * Use is subject to license conditions.
 * 
 * The main licensing options available are:
 * 
 * Open Source Licensing. This is the appropriate option if you want to
 * share the source code of your application with everyone you
 * distribute it to, and you also want to give them the right to share
 * who uses it. If you wish to use TNC@FHH under Open Source Licensing,
 * you must contribute all your source code to the open source
 * community in accordance with the GPL Version 2 when your application
 * is distributed. See http://www.gnu.org/licenses/gpl-2.0.html/ or gpl-2.0.txt
 * 
 * Commercial Licensing. This is the appropriate option if you are
 * creating proprietary applications and you are not prepared to
 * distribute and share the source code of your application.
 * Contact trust@f4-i.fh-hannover.de for details.
 * 
 * http://trust.inform.fh-hannover.de/
 * 
 */

package de.fhhannover.inform.trust.xacml;

import java.io.File;
import java.io.FilenameFilter;
import java.io.IOException;
import java.net.URISyntaxException;

import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;

import com.sun.xacml.UnknownIdentifierException;

/**
 * 
 * 
 * @author Bastian Hellmann
 */
public class StaticNetworkPDPStarter {

	private static Logger logger = Logger
			.getLogger(StaticNetworkPDPStarter.class);

	/**
	 * @param args
	 * @throws Exception
	 */
	public static void main(String[] args) {
		PropertyConfigurator.configure("log4j.properties");

		if (args.length < 2) {
			System.err.println("Wrong number of arguments");
			System.err.println("Usage:");
			System.err
					.println("StaticNetworkPDPStarter <port> <policy file #1> <policy file #2> ... <policy file #n>");
			System.err
					.println("or                      <port> <filename postfix, e.g. '.xml'> <path to policy files (must end with defined filename postfix')>");
			System.exit(1);
		}

		int port = Integer.parseInt(args[0]);
		String[] policyFiles = new String[args.length - 1];

		logger.info("Trust@FHH XACML PDP with static policy handling (only loading during startup) starts.");

		if (args.length == 3 && new File(args[2]).isDirectory()) {
			File policyDirectory = new File(args[2]);
			final String filenamePostfix = args[1].replace(".", "");
			logger.info("Loading all files ending with '." + filenamePostfix
					+ "' from directory " + policyDirectory);
			policyFiles = policyDirectory.list(new FilenameFilter() {
				@Override
				public boolean accept(File dir, String name) {
					if (name.toLowerCase().endsWith(filenamePostfix)) {
						return true;
					} else {
						return false;
					}
				}
			});

			for (int i = 0; i < policyFiles.length; i++) {
				try {
					policyFiles[i] = policyDirectory.getCanonicalPath() + "/"
							+ policyFiles[i];
				} catch (IOException e) {
					logger.error("'IOException' at loading file: "
							+ e.getMessage());
				}
			}
		} else {
			logger.info("Loading all given policy files");
			for (int i = 1; i < args.length; i++) {
				policyFiles[i - 1] = args[i];
			}
		}

		for (int i = 0; i < policyFiles.length; i++) {
			logger.info("Policy file #" + (i + 1) + "/" + policyFiles.length
					+ ": " + policyFiles[i]);
		}

		NetworkPDP pdp;
		try {
			pdp = new StaticNetworkPDP(policyFiles);

			NetworkPDPDispatcher dispatcher;
			dispatcher = new NetworkPDPDispatcher(pdp, port);

			logger.info("Dispatcher is started.");

			dispatcher.runServer();
		} catch (URISyntaxException e) {
			logger.error("'URISyntaxException' during initialization of StaticNetworkPDP: "
					+ e.getMessage());
			System.exit(1);
		} catch (UnknownIdentifierException e) {
			logger.error("'UnknownIdentifierException' during initialization of StaticNetworkPDP: "
					+ e.getMessage());
			System.exit(1);
		}

	}
}
